import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="shipmnts-pubsub",
    version="0.0.35",
    author="Vimox Shah",
    author_email="vimox@shipmnts.com",
    description="Pubsub handler",
    long_description_content_type="text/markdown",
    install_requires=["google-cloud-pubsub==1.4.3"],
    url="https://github.com/vimox-shah/shipmnts_pubsub",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import logging
import json
from google.cloud import pubsub_v1


retry_settings = {
    "interfaces": {
        "google.pubsub.v1.Publisher": {
            "retry_codes": {
                "publish": [
                    "ABORTED",
                    "CANCELLED",
                    "DEADLINE_EXCEEDED",
                    "INTERNAL",
                    "RESOURCE_EXHAUSTED",
                    "UNAVAILABLE",
                    "UNKNOWN",
                ]
            },
            "retry_params": {
                "messaging": {
                    "initial_retry_delay_millis": 100,  # default: 100
                    "retry_delay_multiplier": 1.3,  # default: 1.3
                    "max_retry_delay_millis": 60000,  # default: 60000
                    "initial_rpc_timeout_millis": 5000,  # default: 25000
                    "rpc_timeout_multiplier": 1.0,  # default: 1.0
                    "max_rpc_timeout_millis": 600000,  # default: 30000
                    "total_timeout_millis": 600000,  # default: 600000
                }
            },
            "methods": {
                "Publish": {
                    "retry_codes_name": "publish",
                    "retry_params_name": "messaging",
                }
            },
        }
    }
}
"""
Retry settings control both the total number of retries and exponential backoff (how long the client waits between subsequent retries). The initial RPC timeout is the time the client waits for the initial RPC to succeed before retrying. The total timeout is the time the client waits before it stops retrying. To retry publish requests, the initial RPC timeout should be shorter than the total timeout.

Once the first RPC fails or times out, the exponential backoff logic determines when the subsequent retries occur. On each retry, the RPC timeout increases by this multiplier, up to the maximum RPC timeout. In addition, the retry delay settings determine how long the client waits between getting an error or timeout and initiating the next request.
"""

publisher = pubsub_v1.PublisherClient(client_config=retry_settings)
def publish_event(data, project_id, topic_name):
    """Publishes multiple messages to a Pub/Sub topic with an error handler."""
    topic_path = publisher.topic_path(project_id, topic_name)
    logging.info("{} Topic Path".format(topic_path))

    def callback(message_future):
        # When timeout is unspecified, the exception method waits indefinitely.
        if message_future.exception(timeout=30):
            logging.info(
                "Publishing message on {} threw an Exception {}.".format(
                    topic_name, message_future.exception()
                )
            )
        else:
            logging.info(message_future.result())

    logging.info("Published message IDs:")
    data = json.dumps(data).encode("utf-8")
    message_future = publisher.publish(topic_path, data=data)
    message_future.add_done_callback(callback)
    message_future.result()
 


import logging
import os
import json
import hashlib
from redis import StrictRedis
from pb.error_event import publish_to_error
from rest_framework.response import Response
from django.db import connection
from rest_framework import status

redis_host = os.getenv('REDIS_HOST', 'redis://localhost')
redis = StrictRedis.from_url(redis_host)


def handle_error(data, counter, is_processed, project_id, topic_name=None):
    retry_limit_exceeded = False
    if is_processed:
        logging.info("Message is already processed")
        return True
    if counter >= 1:
        logging.info(
            "Retry limit exceeded. Currently Retry changed from 5 to 0")
        logging.info("data in handle error %s", data)
        try:
            with connection.cursor() as cursor:
                cursor.execute(
                    "UPDATE jobs set status = 'failed' WHERE id={0}".format(
                        data['job_id']
                    )
                )
                if data.get("child_document_id"):
                    cursor.execute("UPDATE child_documents set status = 'failed' WHERE id={0}".format(
                        data.get("child_document_id")))
            if topic_name:
                publish_to_error(data={'job_id': data['job_id'], 'tenant': data['tenant'],
                                       'service': data.get("service")}, project_id=project_id, topic_name=topic_name)
            retry_limit_exceeded = True
        except Exception as e:
            logging.info(
                "Job id or tenant is missing in data. Exception = {0}".format(e))
        finally:
            return retry_limit_exceeded


def create_key(request, data):
    # This helper function creates a unique key for a message
    str_data = json.dumps(data)
    sha256 = hashlib.sha256(str_data.encode('utf-8')).hexdigest()
    subscription = request['subscription'].split('/')[-1]
    message_id = request['message']['messageId']
    return ["%s_%s_%s" % (subscription, sha256, message_id), "%s_%s" % (subscription, message_id)]


def get_count(key):
    # In case you want to wait some arbitrary time before your message "fails"
    counter = redis.get(key)
    if counter:
        redis.incr(key)
        counter = redis.get(key)
    else:
        counter = 0
        redis.set(key, counter, 3600*6)
    return int(counter)


def check_message_processed(status_key):
    status = redis.get(status_key)
    if status:
        is_processed = True
    else:
        is_processed = False
    return is_processed


def set_message_status(request):
    subscription = request['subscription'].split('/')[-1]
    message_id = request['message']['messageId']
    key = "%s_%s" % (subscription, message_id)
    redis.set(key, "1", 3600*6)


def handle_pubsub_retry(request, data, project_id, topic_name=None):
    key, msg_status_key = create_key(request, data)
    counter = get_count(key)
    is_processed = check_message_processed(msg_status_key)
    logging.info("message key for retry %s, counter %s, is_processed: %s",
                 key, counter, is_processed)
    return handle_error(data, counter, is_processed, project_id, topic_name=topic_name)
